@@ -12,9 +12,9 @@ require Exporter ;
 our @ISA = qw(Exporter) ;
 our %EXPORT_TAGS = ('all' => [ qw() ]) ;
 our @EXPORT_OK = ( @{$EXPORT_TAGS{'all'} } ) ;
-our @EXPORT = qw(DumpTree PrintTree DumpTrees CreateChainingFilter);
+our @EXPORT = qw(DumpTree PrintTree DumpTrees CreateChainingFilter MD1 MD2) ;
 
-our $VERSION = '0.39' ;
+our $VERSION = '0.41' ;
 
 my $WIN32_CONSOLE ;
 
@@ -34,10 +34,14 @@ BEGIN
 		}
 	}
 	
-use Text::Wrap  ;
+use Text::Wrap ;
+#use Text::ANSI::Util qw(ta_wrap);
 use Class::ISA ;
 use Sort::Naturally ;
 
+use constant MD1 => ('MAX_DEPTH' => 1) ;
+use constant MD2 => ('MAX_DEPTH' => 2) ;
+
 #-------------------------------------------------------------------------------
 # setup values
 #-------------------------------------------------------------------------------
@@ -53,7 +57,7 @@ our %setup =
 	, INDENTATION            => ''
 	, NO_OUTPUT              => 0
 	, START_LEVEL            => 1
-	, VIRTUAL_WIDTH          => 120
+	, VIRTUAL_WIDTH          => 160
 	, DISPLAY_ROOT_ADDRESS   => 0
 	, DISPLAY_ADDRESS        => 1
 	, DISPLAY_PATH           => 0
@@ -67,11 +71,12 @@ our %setup =
 	, COLOR_LEVELS           => undef
 	, GLYPHS                 => ['|  ', '|- ', '`- ', '   ']
 	, QUOTE_HASH_KEYS        => 0
+	, DISPLAY_NO_VALUE	 => 0
 	, QUOTE_VALUES           => 0
 	, REPLACEMENT_LIST       => [ ["\n" => '[\n]'], ["\r" => '[\r]'], ["\t" => '[\t]'] ]
 	
 	, DISPLAY_NUMBER_OF_ELEMENTS_OVER_MAX_DEPTH => 0
-	
+	, ELEMENT                => 'element'
 	, DISPLAY_CALLER_LOCATION=> 0
 	
 	, __DATA_PATH            => ''
@@ -114,8 +119,10 @@ our $Numberlevels         = $setup{NUMBER_LEVELS} ;
 our $Colorlevels          = $setup{COLOR_LEVELS} ;
 our $Glyphs               = [@{$setup{GLYPHS}}] ; # we don't want it to be shared
 our $Quotehashkeys        = $setup{QUOTE_HASH} ;
+our $Displaynovalue       = $setup{DISPLAY_NO_VALUE} ;
 our $Quotevalues          = $setup{QUOTE_VALUES} ;
 our $ReplacementList      = [@{$setup{REPLACEMENT_LIST}}] ; # we don't want it to be shared
+our $Element              = $setup{ELEMENT} ;
 
 our $Displaynumberofelementsovermaxdepth = $setup{DISPLAY_NUMBER_OF_ELEMENTS_OVER_MAX_DEPTH} ;
 
@@ -150,6 +157,7 @@ return
 	, GLYPHS                 => $Data::TreeDumper::Glyphs
 	, QUOTE_HASH_KEYS        => $Data::TreeDumper::Quotehashkeys
 	, REPLACEMENT_LIST       => $Data::TreeDumper::ReplacementList
+	, ELEMENT                => $Data::TreeDumper::Element
 	
 	, DISPLAY_NUMBER_OF_ELEMENTS_OVER_MAX_DEPTH => $Displaynumberofelementsovermaxdepth
 	
@@ -167,41 +175,28 @@ return
 
 sub PrintTree
 {
-my ($package, $file_name, $line) = caller() ;
-print DumpTree(@_, DUMPER_NAME => "PrintTree  at '$file_name:$line'") ;
+print DumpTree(@_) ;
 }
 
 sub DumpTree
 {
 my $structure_to_dump = shift ;
-my $title             = shift ;
-my %overrides         =  @_ ;
-
-$title = defined $title ? $title : '' ;
+my $title             = shift // '' ;
 
 my ($package, $file_name, $line) = caller() ;
 
-my $location = '' ;
+die "Error: Odd number of arguments @ $file_name:$line\n" if @_ % 2 ; 
 
-if($Displaycallerlocation)
-	{
-	$location = defined $overrides{DUMPER_NAME} ? $overrides{DUMPER_NAME} : "DumpTree at '$file_name:$line'" ;
-	}
-	
-unless(defined $structure_to_dump)
-	{
-	return("$title (undefined variable) $location\n") ;
-	}
+my %overrides = @_ ;
 
-if('' eq ref $structure_to_dump)
-	{
-	return("$title $structure_to_dump (scalar variable) $location\n");
-	}
+$overrides{DUMPER_NAME} //= "DumpTree $file_name:$line" ;
+my $location = $overrides{DUMPER_NAME} ;
+
+return "$title (undefined variable) $location\n" unless defined $structure_to_dump ;
+
+return "$title $structure_to_dump (scalar variable) @ $location\n" if '' eq ref $structure_to_dump ;
 	
-if($Displaycallerlocation)
-	{
-	print "$location\n" ;
-	}
+print STDERR "$location\n" if $Displaycallerlocation ;
 
 my %local_setup ;
 
@@ -216,7 +211,7 @@ else
 	
 unless (exists $local_setup{TYPE_FILTERS}{Regexp})
 	{
-	# regexp objecjts (created with qr) are dumped by the below sub
+	# regexp objects (created with qr) are dumped by the below sub
 	$local_setup{TYPE_FILTERS}{Regexp} =
 		sub
 		{
@@ -225,7 +220,7 @@ unless (exists $local_setup{TYPE_FILTERS}{Regexp})
 		} ;
 	}
 	
-return(TreeDumper($structure_to_dump, {TITLE => $title, %local_setup})) ;
+return TreeDumper($structure_to_dump, {TITLE => $title, %local_setup}) ;
 }
 
 #-------------------------------------------------------------------------------
@@ -249,7 +244,11 @@ for my $tree (@trees)
 	else
 		{
 		my ($package, $file_name, $line) = caller() ;
-		$dump .= "DumpTrees can't dump 'undef' with title: '$title' @ '$file_name:$line'.\n" ;
+
+		$global_overrides{DUMPER_NAME} //= "DumpTree @ $file_name:$line" ;
+		my $location = $global_overrides{DUMPER_NAME} ;
+
+		$dump .= "Can't dump 'undef' with title: '$title' @ $location.\n" ;
 		}
 	}
 	
@@ -418,9 +417,7 @@ $filter_sub = $type_filters->{$type} if(defined $type_filters && exists $type_fi
 
 unless ('CODE' eq ref $filter_sub || ! defined $filter_sub)
 	{
-	my ($package, $file_name, $line) = caller(2) ;
-	
-	die "DumpTree: FILTER must be sub reference at '$file_name:$line'" ;
+	die "FILTER must be sub reference at $setup->{DUMPER_NAME}\n" ;
 	}
 
 return($filter_sub, $setup->{FILTER_ARGUMENT}) ;
@@ -458,15 +455,6 @@ for($tree)
 	
 	($tree_type eq 'ARRAY' || obj($tree, 'ARRAY')) and do
 		{
-		#~ # debug while writting Diff module
-		#~ unless(defined $nodes_to_display->[$node_index])
-			#~ {
-			#~ use Data::Dumper ;
-			#~ print Dumper $nodes_to_display ;
-			#~ my ($package, $file_name, $line) = caller() ;
-			#~ print "Called from $file_name, $line\n" ;
-			#~ print "$tree->\[$nodes_to_display->\[$node_index\]\]\n" ;
-			#~ }
 		$element = $tree->[$nodes_to_display->[$node_index]] ;
 		$element_address = "$element" if defined $element ;
 		$element_name = $node_names->[$node_index] ;
@@ -585,7 +573,7 @@ my $output = RenderNode
 return($output) ;
 }
 
-#-------------------------------------------------------------------------------
+#------------------------------------------------------------------------------
 
 sub GetBrackets
 {
@@ -674,14 +662,18 @@ unless($setup->{NO_OUTPUT})
 		}
 	else
 		{
-		$output .= $setup->{INDENTATION} ;
-		
-		$output .= defined $setup->{TITLE} ? $setup->{TITLE} : '' ;
-		$output .= $root_tie_and_class ;
-		$output .= ' [' . GetReferenceType($tree) . "0]" if($setup->{DISPLAY_ROOT_ADDRESS}) ;
-		$output .= " $tree"                              if($setup->{DISPLAY_PERL_ADDRESS}) ;
-		$output .= " <" . total_size($tree) . ">"        if($setup->{DISPLAY_PERL_SIZE}) ;
-		$output .= "\n" ;
+		my $title = $setup->{TITLE} // '' ;
+
+		if($title ne q{})
+			{
+			$output .= $setup->{INDENTATION}
+					. $title
+					. $root_tie_and_class
+					. ($setup->{DISPLAY_ADDRESS} && $setup->{DISPLAY_ROOT_ADDRESS} ?' [' . GetReferenceType($tree) . "0]" : '')
+					. ($setup->{DISPLAY_PERL_ADDRESS} ? " $tree" : '')
+					. ($setup->{DISPLAY_PERL_SIZE}    ? " <" . total_size($tree) . ">" : '')
+					. "\n" ;
+			}
 		}
 	}
 	
@@ -783,23 +775,42 @@ else
 					}
 				}
 				
-			if($columns eq '')
+			if(!defined $columns || $columns eq '')
 				{
 				$columns = $setup->{VIRTUAL_WIDTH}  ;
 				}
 			}
 			
-		local $Text::Wrap::columns  = $columns ;
+		#use Text::ANSI::Util 'ta_wrap' ;
+		#my $header_length = ta_length($tree_header . $tree_subsequent_header) ;
+ 
+		local $Text::Wrap::columns  = 400 ;
 		local $Text::Wrap::unexpand = 0 ;
+		local $Text::Wrap::tabstop = 1 ;
+		local $Text::Wrap::huge = 'overflow' ;
 		
 		if(length($tree_header) + length($element_description) > $columns && ! $setup->{NO_WRAP})
 			{
+			#$output .= ta_wrap
+			#		(
+			#		$element_description,
+			#		$columns,
+			#		{
+			#			flindent => $tree_header, 
+			#			slindent => $tree_subsequent_header, 
+			#		}
+			#		) ;
+
+			$output .= $tree_header ;
+			$output .= $element_description ;
+=pod
 			$output .= wrap
 					(
 					  $tree_header 
 					, $tree_subsequent_header 
 					, $element_description
 					) ;
+=cut
 			}
 		else
 			{
@@ -859,16 +870,19 @@ for(ref $element)
 				$element_value =~ s/$find/$replace/g ;
 				}
 			}
-		
-		if($setup->{QUOTE_VALUES} && defined $element)
-			{
-			$default_element_rendering = " = '$element_value'" ;
+	
+		unless ($setup->{DISPLAY_NO_VALUE})
+			{	
+			if($setup->{QUOTE_VALUES} && defined $element)
+				{
+				$default_element_rendering = " = '$element_value'" ;
+				}
+			else
+				{
+				$default_element_rendering = " = $element_value" ;
+				}
 			}
-		else
-			{
-			$default_element_rendering = " = $element_value" ;
-			}
-			
+	
 		$perl_address = "$element_id" if($setup->{DISPLAY_PERL_ADDRESS}) ;
 		
 		# $setup->{DISPLAY_TIE} doesn't make sense as scalars are copied
@@ -890,7 +904,7 @@ for(ref $element)
 		
 		if(! %{$element} && ! $setup->{NO_NO_ELEMENTS})
 			{
-			$default_element_rendering = $element_value = ' (no elements)' ;
+			$default_element_rendering = $element_value = ' (no ' . $setup->{ELEMENT} . 's)' ;
 			}
 		
 		if
@@ -905,7 +919,7 @@ for(ref $element)
 			{
 			my $number_of_elements = keys %{$element} ;
 			my $plural = $number_of_elements > 1 ? 's' : '' ;
-			my $elements = ' (' . $number_of_elements . ' element' . $plural . ')' ; 
+			my $elements = ' (' . $number_of_elements . ' ' . $setup->{ELEMENT} . $plural . ')' ; 
 			
 			$default_element_rendering .= $elements ;
 			$element_value .= $elements ;
@@ -937,7 +951,7 @@ for(ref $element)
 		
 		if(! @{$element} && ! $setup->{NO_NO_ELEMENTS})
 			{
-			$default_element_rendering = $element_value .= ' (no elements)' ;
+			$default_element_rendering = $element_value .= ' (no ' . $setup->{ELEMENT} . 's)' ;
 			}
 		
 		if
@@ -950,8 +964,8 @@ for(ref $element)
 				)
 			)
 			{
-			my $plural = scalar(@{$element}) ? 's' : '' ;
-			my $elements = ' (' . @{$element} . ' element' . $plural . ')' ; 
+			my $plural = scalar(@{$element}) > 1 ? 's' : '' ;
+			my $elements = ' (' . @{$element} . ' ' . $setup->{ELEMENT} . $plural . ')' ; 
 			
 			$default_element_rendering .= $elements ;
 			$element_value .= $elements ;
@@ -1025,7 +1039,7 @@ for(ref $element)
 			{
 			my $number_of_elements = keys %{$element} ;
 			my $plural = $number_of_elements > 1 ? 's' : '' ;
-			$object_elements = ' (' . $number_of_elements . ' element' . $plural . ')' ; 
+			$object_elements = ' (' . $number_of_elements . ' ' . $setup->{ELEMENT} . $plural . ')' ; 
 			}
 		}
 	elsif(obj($element, 'ARRAY'))
@@ -1042,7 +1056,7 @@ for(ref $element)
 			)
 			{
 			my $plural = scalar(@{$element}) ? 's' : '' ;
-			$object_elements = ' (' . @{$element} . ' element' . $plural . ')' ; 
+			$object_elements = ' (' . @{$element} . ' ' . $setup->{ELEMENT} . $plural . ')' ; 
 			}
 		}
 	elsif(obj($element, 'GLOB'))
@@ -1286,7 +1300,9 @@ for(ref $element)
 	if($setup->{DISPLAY_OBJECT_TYPE})
 		{
 		my $class = ref($element) ;
-		my $has_autoloadd = $class->can("AUTOLOAD") ? '[AL]' : '' ;
+		my $has_autoload ;
+		eval "\$has_autoload = *${class}::AUTOLOAD{CODE} ;" ;
+		$has_autoload = $has_autoload ? '[AL]' : '' ;
 		
 		$element_type .= " blessed in '$has_autoload$class'" ;
 		
@@ -1874,6 +1890,11 @@ the hash keys. Hash keys are not quoted by default.
      |  |- 'b' [H4]
      |  |  |- 'a' = 0 [S5]
 
+
+=head2 DISPLAY_NO_VALUE
+
+Only element names are added to the tree rendering
+
 =head2 QUOTE_VALUES
 
 B<QUOTE_VALUES> and its package variable B<$Data::TreeDumper::Quotevalues> can be set if you wish to single quote
@@ -2452,7 +2473,6 @@ B<Data::TreeDumper> on this track. Check B<Data::TreeDumper::Renderer::DHTML>.
 to setup the document (ex:: html header).
 
 =over 4
-
 my ($title, $type_address, $element, $size, $perl_address, $setup) = @_ ;
 
 =item 1 $title
@@ -2583,6 +2603,8 @@ B<VIRTUAL_WIDTH> instead. Default is 120.
 
 =item * QUOTE_HASH_KEYS
 
+=item * DISPLAY_NO_VALUE
+
 =item * QUOTE_VALUES
 
 =item * REPLACEMENT_LIST
@@ -2717,7 +2739,7 @@ Khemir Nadim ibn Hamouda. <nadim@khemir.net>
 
 Thanks to Ed Avis for showing interest and pushing me to re-write the documentation.
 
-  Copyright (c) 2003-2010 Nadim Ibn Hamouda el Khemir. All rights
+  Copyright (c) 2003-2006 Nadim Ibn Hamouda el Khemir. All rights
   reserved.  This program is free software; you can redis-
   tribute it and/or modify it under the same terms as Perl
   itself.
